import pytest
import os, sys
from unittest.mock import patch, MagicMock
from dotenv import load_dotenv
load_dotenv()

import os

# baichuan
os.environ['BAICHUAN_API_KEY'] = 'your-baichuan-api-key'

# zhipu
os.environ['ZHIPU_API_KEY'] = 'your-zhipu-api-key'

# moonshot
os.environ['MOONSHOT_API_KEY'] = 'your-moonshot-api-key'

# qwen
os.environ['DASHSCOPE_API_KEY'] = 'your-dashscope-api-key'

# minimax
os.environ['MINIMAX_API_KEY'] = 'your-minimax-api-key'

# coze
os.environ['COZE_API_KEY'] = 'your-coze-api-key'
os.environ['COZE_BOT_ID'] = 'your-coze-bot-id'

# dify
os.environ['DIFY_API_KEY'] = 'your-dify-api-key'

# fastgpt
os.environ['FASTGPT_API_KEY'] = 'your-fastgpt-api-key'

# wenxin
os.environ['ERNIE_CLIENT_ID'] = 'your-ernie-client-id'
os.environ['ERNIE_CLIENT_SECRET'] = 'your-ernie-client-secret'

# tiangong
os.environ['TIANGONG_APP_KEY'] = 'your-tiangong-app-key'
os.environ['TIANGONG_APP_SECRET'] = 'your-tiangong-app-secret'

# xunfei
os.environ['XUNFEI_APP_ID'] = 'your-xunfei-app-id'
os.environ['XUNFEI_API_KEY'] = 'your-xunfei-api-key'
os.environ['XUNFEI_API_SECRET'] = 'your-xunfei-api-secret'

# openai
os.environ['OPENAI_API_KEY'] = 'your-openai-api-key'

# azure
os.environ['AZURE_API_KEY'] = 'your-azure-api-key'
os.environ['AZURE_API_BASE'] = 'your-azure-api-base'
os.environ['AZURE_API_VERSION'] = 'your-azure-api-version'

# groq
os.environ['GROQ_API_KEY'] = 'your-groq-api-key'

# bedrock
os.environ['AWS_ACCESS_KEY_ID'] = 'your-aws-access-key-id'
os.environ['AWS_SECRET_ACCESS_KEY'] = 'your-aws-secret-access-key'
os.environ['AWS_REGION_NAME'] = 'your-aws-region-name'

# mistral
os.environ['MISTRAL_API_KEY'] = 'your-mistral-api-key'

# cohere
os.environ['COHERE_API_KEY'] = 'your-cohere-api-key'

# gemini
os.environ['GEMINI_API_KEY'] = 'your-gemini-api-key'

# openai
os.environ['OPENAI_API_KEY'] = 'your-openai-api-key'

# anthropic
os.environ['ANTHROPIC_API_KEY'] = 'your-anthropic-api-key'

# 01 yi-large
os.environ['LINGYI_API_KEY'] = 'your-lingyi-api-key'

# jieyuexingchen step-1-8k
os.environ['STEPFUN_API_KEY'] = 'your-stepfun-api-key'

# doubao
os.environ['ARK_API_KEY'] = 'your-ark-api-key'


# 将项目根目录添加到sys.path中
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from unionllm.exceptions import ProviderError
from unionllm import unionchat
from unionllm.utils import ModelResponse, Message, Choices, Usage, Context

common_messages = [{"content": "你的开发者是谁？", "role": "user"}]

@pytest.mark.parametrize("provider, model, messages, expected_exception, stream_mode", [
    # non stream mode
    # This line is part of a parameterized test in Python using pytest. It defines a set of input
    # parameters for the test function `test_unionchat`.
    # ("azure", "azure/gpt-35-turbo", common_messages, None, False),
    # ("zhipuai", "glm-4", common_messages, None, False),
    # ("tiangong", "SkyChat-MegaVerse", common_messages, None, False),
    # ("wenxin", "ERNIE-3.5-8K", common_messages, None, False),
    # ("qwen", "qwen-plus", common_messages, None, False),
    # ("doubao", "ep-20240630165653-phz9t", common_messages, None, False),
    # ("moonshot", "moonshot-v1-8k", common_messages, None, False),
    # ("minimax", "abab5.5-chat", common_messages, None, False),
    # ("baichuan", "Baichuan2-Turbo", common_messages, None, False),
    # ("xunfei", "generalv3", common_messages, None, False),
    # ("lingyi", "yi-large", common_messages, None, False),
    # ("stepfun", "step-1-8k", common_messages, None, False),
    # ("coze", "coze", common_messages, None, False),
    # ("dify", "dify", common_messages, None, False),
    # ("fastgpt", "fastgpt", common_messages, None, False),    
    # {"ollama", "ollama/llama3", common_messages, None, False}
    # ("mistral", "mistral-large-latest", common_messages, None, False),
    # ("cohere", "command-r", common_messages, None, False),
    # ("groq", "groq/mixtral-8x7b-32768", common_messages, None, False),
    # ("bedrock", "ai21.j2-ultra-v1", common_messages, None, False),
    # ("", "groq/mixtral-8x7b-32768", common_messages, None, False), #no provider case
    # ("nonexistent", "model", common_messages, ProviderError, False),
    # stream mode
    # ("doubao", "ep-20240630165653-phz9t", common_messages, None, True),
    # ("azure", "azure/gpt-35-turbo", common_messages, None, True),
    # ("zhipuai", "glm-4", common_messages, None, True),
    # ("nonexistent", "model", common_messages, ProviderError, True),
    # ("tiangong", "SkyChat-MegaVerse", common_messages, None, True),
    # ("wenxin", "ERNIE-3.5-8K", common_messages, None, True),
    # ("qwen", "qwen-plus", common_messages, None, True),
    # ("moonshot", "moonshot-v1-8k", common_messages, None, True),
    # ("minimax", "abab5.5-chat", common_messages, None, True),
    # ("baichuan", "Baichuan2-Turbo", common_messages, None, True),
    # ("coze", "coze", common_messages, None, True),
    # ("dify", "dify", common_messages, None, True),
    # ("fastgpt", "fastgpt", common_messages, None, True),    
    # {"ollama", "ollama/llama3", common_messages, None, False}
    # ("mistral", "mistral-large-latest", common_messages, None, True),
    # ("cohere", "command-r", common_messages, None, True),
    # ("groq", "groq/mixtral-8x7b-32768", common_messages, None, True),
    # ("bedrock", "ai21.j2-ultra-v1", common_messages, None, True),
    # ("nonexistent", "model", common_messages, ProviderError, True),
    # ("lingyi", "yi-large", common_messages, None, True),
    # ("stepfun", "step-1-8k", common_messages, None, True),
])

def test_unionchat(provider, model, messages, expected_exception, stream_mode, mocker):
    mock_unionllm = mocker.patch('unionllm.UnionLLM')
    if expected_exception:
        mock_unionllm.return_value.completion.side_effect = ProviderError("Provider not supported")
    else:
        if stream_mode:
            def stream_response():
                yield ModelResponse(id='8635254124951169203', choices=[Choices(finish_reason='stop', index=0, message=Message(content='我是人工智能助手。', role='assistant'))], created=1715570856, model=model, object='chat.completion', system_fingerprint=None, usage=Usage(prompt_tokens=9, completion_tokens=27, total_tokens=36))
            mock_unionllm.return_value.completion.return_value = stream_response()
        else:
            mock_unionllm.return_value.completion.return_value = ModelResponse(id='8635254124951169203', choices=[Choices(finish_reason='stop', index=0, message=Message(content='我是人工智能助手。', role='assistant'))], created=1715570856, model=model, object='chat.completion', system_fingerprint=None, usage=Usage(prompt_tokens=9, completion_tokens=27, total_tokens=36))

    if expected_exception:
        with pytest.raises(expected_exception):
            unionchat(provider=provider, model=model, messages=messages, stream=stream_mode)
    else:
        result = unionchat(provider=provider, model=model, messages=messages, stream=stream_mode)
        if stream_mode:
            for response in result:
                # print(response)
                assert isinstance(response, ModelResponse), "Each item in the stream should be an instance of ModelResponse"
        else:
            # print(result)
            assert isinstance(result, ModelResponse), "The result should be an instance of ModelResponse"


from .base_provider import BaseProvider
from openai import OpenAI
import logging, json, os


class DeepSeekError(Exception):
    def __init__(
        self,
        status_code,
        message,
    ):
        self.status_code = status_code
        self.message = message
        super().__init__(self.message)


class DeepSeekAIProvider(BaseProvider):
    def __init__(self, **model_kwargs):
        # Get MOONSHOT_API_KEY from environment variables
        _env_api_key = os.environ.get("DEEPSEEK_API_KEY")
        self.api_key = model_kwargs.get("api_key") if model_kwargs.get("api_key") else _env_api_key
        if not self.api_key:
            raise DeepSeekError(
                status_code=422, message=f"Missing API key"
            )
        
        if model_kwargs.get("beta"):
            self.base_url = "https://api.deepseek.com/beta"
        else:
            self.base_url = "https://api.deepseek.com/v1"
        self.client = OpenAI(api_key=self.api_key, base_url=self.base_url)

    def pre_processing(self, **kwargs):
        # process the compatibility issue of parameters, all unsupported parameters are discarded
        supported_params = [
            "model", "messages", "max_tokens", "temperature", "top_p", "n",
            "logprobs", "stream", "stop", "presence_penalty", "frequency_penalty",
            "best_of", "logit_bias"
        ]
        for key in list(kwargs.keys()):
            if key not in supported_params:
                kwargs.pop(key)

        return kwargs
    
    def post_stream_processing_wrapper(self, model, messages, **new_kwargs):
        result = self.client.chat.completions.create(
            model=model, messages=messages, **new_kwargs
        )
        return self.post_stream_processing(result, model=model)

    def create_model_response_wrapper(self, result, model):
        # 调用 response_model 中的 create_model_response 方法
        return self.create_model_response(result, model=model)

    def completion(self, model: str, messages: list, **kwargs):
        try:
            if model is None or messages is None:
                raise DeepSeekError(
                    status_code=422, message=f"Missing model or messages"
                )
            message_check_result = self.check_prompt("deepseek", model, messages)            
            if message_check_result['pass_check']:
                messages = message_check_result['messages']
            else:
                raise DeepSeekError(
                    status_code=422, message=message_check_result['reason']
                )
            new_kwargs = self.pre_processing(**kwargs)
            stream = kwargs.get("stream", False)

            if stream:
                return self.post_stream_processing_wrapper(model=model, messages=messages, **new_kwargs)
            else:
                result = self.client.chat.completions.create(
                    model=model, messages=messages, **new_kwargs
                )
                return self.create_model_response_wrapper(result, model=model)
        except Exception as e:
            if hasattr(e, "status_code"):
                raise DeepSeekError(status_code=e.status_code, message=str(e))
            else:
                raise DeepSeekError(status_code=500, message=str(e))



import time, os
import requests
import json
import hashlib
from .base_provider import BaseProvider
from unionllm.utils import ModelResponse, Message, Choices, Usage, Delta, StreamingChoices

class TianGongOpenAIError(Exception):
    def __init__(
        self,
        status_code,
        message,
    ):
        self.status_code = status_code
        self.message = message
        super().__init__(self.message)

class TianGongAIProvider(BaseProvider):
    def __init__(self, **model_kwargs):
        # Get TIANGONG_API_KEY from environment variables
        _env_app_key = os.environ.get("TIANGONG_APP_KEY")
        _env_app_secret = os.environ.get("TIANGONG_APP_SECRET")
        self.app_key = model_kwargs.get("app_key") if model_kwargs.get("app_key") else _env_app_key
        self.app_secret = model_kwargs.get("app_secret") if model_kwargs.get("app_secret") else _env_app_secret
        if not self.app_key or not self.app_secret:
            raise TianGongOpenAIError(
                status_code=422, message=f"Missing APP key or APP secret"
            )
        self.endpoint_url = "https://sky-api.singularity-ai.com/saas/api/v4/generate"

    def pre_processing(self, **kwargs):
        supported_params = [
            "model", "messages", "max_tokens", "temperature", "top_p", "n",
            "logprobs", "stream", "stop", "presence_penalty", "frequency_penalty",
            "best_of", "logit_bias"
        ]
        for key in list(kwargs.keys()):
            if key not in supported_params:
                kwargs.pop(key)
        return kwargs

    def to_formatted_prompt(self, messages):
        # replace the role of assistant to bot
        for message in messages:
            if message.get("role") == "assistant":
                message["role"] = "bot"
        return messages
    
    def post_stream_processing_wrapper(self, model, messages, **new_kwargs):
        timestamp = str(int(time.time()))
        sign_content = self.app_key + self.app_secret + timestamp
        sign_result = hashlib.md5(sign_content.encode("utf-8")).hexdigest()
        payload = {"model": model, "messages": messages, **new_kwargs}
        headers = {
            "app_key": self.app_key,
            "timestamp": timestamp,
            "sign": sign_result,
            "Content-Type": "application/json",
            "stream": "true",
        }
        result = requests.post(self.endpoint_url, headers=headers, json=payload, stream=True)
        index = 0
        for line in result.iter_lines():
            if line:
                new_line = json.loads(line.decode('utf-8'))
                chunk_choices = []
                chunk_delta = Delta()
                if 'reply' in new_line["resp_data"]:
                    chunk_delta.role = "assistant"
                    chunk_delta.content=new_line["resp_data"]["reply"]
                    chunk_choices.append(StreamingChoices(index=index, delta=chunk_delta))

                if 'usage' in new_line["resp_data"]:
                    chunk_usage = Usage()
                    if "input_tokens" in new_line["resp_data"]["usage"]:
                        chunk_usage.prompt_tokens = new_line["resp_data"]["usage"]["prompt_tokens"],
                    if "output_tokens" in new_line["resp_data"]["usage"]:
                        chunk_usage.completion_tokens = new_line["resp_data"]["usage"]["output_tokens"]
                    if "total_tokens" in new_line["resp_data"]["usage"]:
                        chunk_usage.total_tokens = new_line["resp_data"]["usage"]["total_tokens"]
                else:
                    chunk_usage = None

                chunk_response = ModelResponse(
                    id=new_line['trace_id'],
                    choices=chunk_choices,
                    created=int(time.time()),
                    model=model,
                    usage=chunk_usage if chunk_usage else None,
                    stream=True
                )
                index += 1
                yield chunk_response

    def create_model_response_wrapper(self, result, model):
        response_dict = result.json()
        choices = []
        message = Message(content=response_dict["resp_data"]["reply"], role="assistant")
        choices.append(
            Choices(
                message=message,
                index=0,
                finish_reason=response_dict["resp_data"]["finish_reason"],
            )
        )
        usage = Usage(
            prompt_tokens=response_dict["resp_data"]["usage"]["prompt_tokens"],
            completion_tokens=response_dict["resp_data"]["usage"]["completion_tokens"],
            total_tokens=response_dict["resp_data"]["usage"]["total_tokens"],
        )
        response = ModelResponse(
            id=response_dict["trace_id"],
            choices=choices,
            created=int(time.time()),
            model=model,
            usage=usage,
        )
        return response


    def completion(self, model: str, messages: list, **kwargs):
        if model is None or messages is None:
            raise TianGongOpenAIError(
                status_code=422, message=f"Missing model or messages"
            )
               
        message_check_result = self.check_prompt("tiangong", model, messages)            
        if message_check_result['pass_check']:
            messages = message_check_result['messages']
        else:
            raise TianGongOpenAIError(
                status_code=422, message=message_check_result['reason']
            )
            
        new_kwargs = self.pre_processing(**kwargs)
        stream = kwargs.get("stream", False)

        messages = self.to_formatted_prompt(messages)

        if stream:
            return self.post_stream_processing_wrapper(model, messages, **new_kwargs)
        else:
            timestamp = str(int(time.time()))
            sign_content = self.app_key + self.app_secret + timestamp
            sign_result = hashlib.md5(sign_content.encode("utf-8")).hexdigest()
            payload = {"model": model, "messages": messages, **new_kwargs}
            headers = {
                "app_key": self.app_key,
                "timestamp": timestamp,
                "sign": sign_result,
                "Content-Type": "application/json",
                "stream": "false",
            }
            result = requests.post(self.endpoint_url, headers=headers, json=payload)
            result_json = result.json()
            if result_json['code'] != 200:
                raise TianGongOpenAIError(
                    status_code=result_json['code'], message=result_json['code_msg']
                )
            return self.create_model_response_wrapper(result, model=model)



import dashscope
from .base_provider import BaseProvider
from http import HTTPStatus
from dashscope import Generation
from unionllm.utils import ModelResponse, Message, Choices, Usage, Delta, StreamingChoices
import json, time, os

class QwenOpenAIError(Exception):
    def __init__(
        self,
        status_code,
        message,
    ):
        self.status_code = status_code
        self.message = message
        super().__init__(self.message)

class QwenAIProvider(BaseProvider):
    def __init__(self, **model_kwargs):
        # Get DASHSCOPE_API_KEY from environment variables
        _env_api_key = os.environ.get("DASHSCOPE_API_KEY")
        self.api_key = model_kwargs.get("api_key") if model_kwargs.get("api_key") else _env_api_key
        if not self.api_key:
            raise QwenOpenAIError(
                status_code=422, message=f"Missing API key"
            )        
        dashscope.api_key = self.api_key

    def pre_processing(self, **kwargs):
        supported_params = [
            "model", "messages", "max_tokens", "temperature", "top_p", "n",
            "logprobs", "stream", "stop", "presence_penalty", "frequency_penalty",
            "best_of", "logit_bias"
        ]
        for key in list(kwargs.keys()):
            if key not in supported_params:
                kwargs.pop(key)
        return kwargs

    def post_stream_processing_wrapper(self, model, messages, **new_kwargs):
        responses = Generation.call(
            model=model,
            messages=messages,
            result_format="message",
            incremental_output=True,
            **new_kwargs,
        )
        for response in responses:
            if response.status_code == HTTPStatus.OK:
                # chunk_message = response.output.choices[0].message
                chunk_choices = []
                index = 0
                for choice in response.output.choices:
                    chunk_message = choice.message
                    chunk_delta = Delta()
                    if chunk_message:
                        if "role" in chunk_message:
                            chunk_delta.role = chunk_message["role"]
                        if "content" in chunk_message:
                            chunk_delta.content = chunk_message["content"]
                        chunk_choices.append(StreamingChoices(index=str(index), delta=chunk_delta))

                if hasattr(response, "usage") and response.usage is not None:
                    chunk_usage = Usage()
                    if "input_tokens" in response.usage:
                        chunk_usage.prompt_tokens = response.usage["input_tokens"]
                    if "output_tokens" in response.usage:
                        chunk_usage.completion_tokens = response.usage["output_tokens"]
                    if "total_tokens" in response.usage:
                        chunk_usage.total_tokens = response.usage["total_tokens"]

                chunk_response = ModelResponse(
                    id=response.request_id,
                    choices=chunk_choices,
                    created=int(time.time()),
                    model=model,
                    usage=chunk_usage if chunk_usage else None,
                    stream=True
                )
                index += 1
                yield chunk_response

            else:
                raise QwenOpenAIError(
                    status_code=response.status_code,
                    message=f"error message: {response.message}",
                )
                

    def create_model_response_wrapper(self, response, model):
        if response.status_code == HTTPStatus.OK:
            response_dict = response.output
            choices = []

            for index, choice in enumerate(response_dict.choices):
                message = Message(
                    content=choice.message.content,
                    role=choice.message.role
                )
                choices.append(
                    Choices(
                        message=message,
                        index=index,
                        finish_reason=choice.finish_reason,
                    )
                )

            usage = Usage(
                prompt_tokens=response.usage.input_tokens,
                completion_tokens=response.usage.output_tokens,
                total_tokens=response.usage.total_tokens,
            )

            return ModelResponse(
                id=response.request_id,
                choices=choices,
                created=int(time.time()),
                model=model,
                usage=usage,
            )
        else:
            raise QwenOpenAIError(
                status_code=response.status_code,
                message=f"Request failed with status code: {response.status_code}",
            )


    def create_qwen_model_response(result: dict, model: str) -> ModelResponse:
        choices = []
        for index, choice in enumerate(result.output.choices):
            message = Message(content=choice.message.content, role=choice.message.role)
            choices.append(
                Choices(message=message, index=index, finish_reason=choice.finish_reason)
            )
        usage = Usage(
            prompt_tokens=result.usage.input_tokens,
            completion_tokens=result.usage.output_tokens,
            total_tokens=result.usage.total_tokens,
        )
        response = ModelResponse(
            id=result.request_id,
            choices=choices,
            created=int(time.time()),
            model=model,
            usage=usage,
        )
        return response


    def completion(self, model: str, messages: list, **kwargs):
        try:
            if model is None or messages is None:
                raise QwenOpenAIError(
                    status_code=422, message=f"Missing model or messages"
                )

            message_check_result = self.check_prompt("qwen", model, messages)   
            if message_check_result['pass_check']:
                messages = message_check_result['messages']
            else:
                raise QwenOpenAIError(
                    status_code=422, message=message_check_result['reason']
                )                
            new_kwargs = self.pre_processing(**kwargs)
            stream = new_kwargs.get("stream", False)
            if stream:
                return self.post_stream_processing_wrapper(model, messages, **new_kwargs)
            else:
                response = Generation.call(
                    model=model,
                    messages=messages,
                    result_format="message",
                    **new_kwargs,
                )
                if response.status_code == HTTPStatus.OK:
                    return self.create_model_response_wrapper(response, model=model)
                else:
                    return {'success': False, 'error': {'status_code': response.status_code, 'message': response.message}}
        except Exception as e:
            if hasattr(e, "status_code"):
                raise QwenOpenAIError(status_code=e.status_code, message=str(e))
            else:
                raise QwenOpenAIError(status_code=500, message=str(e))
        
